<?php
/**
* XSPF MP3 Player Plugin Class using the Community Builder API
* @package Community Builder
* @author Jeffrey Hill jeffrey.r.hill(at)gmail.com
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
*/

class getMP3Tab extends cbTabHandler {

	function getMP3Tab() {
		$this->cbTabHandler();
	}
	
	/**
	* Includes the default language .php file, and defaults to English.
	* @returns boolean
	*/
	function _getLanguageFile()  {
        global $mosConfig_absolute_path, $mosConfig_lang;
        if (file_exists($mosConfig_absolute_path."/components/com_comprofiler/plugin/user/plug_cbmp3tab/language/".$mosConfig_lang.".php") ) {
            include_once($mosConfig_absolute_path."/components/com_comprofiler/plugin/user/plug_cbmp3tab/language/".$mosConfig_lang.".php");
        } else {
            include_once($mosConfig_absolute_path."/components/com_comprofiler/plugin/user/plug_cbmp3tab/language/english.php");
        }
        return true;
    }
    
	/**
	* Loads XSPF plugin parameters.
	* @param array CB user
	* @returns array Plugin Parameters
	*/
	function _getParams($user)  {
        global $mosConfig_live_site, $mosConfig_absolute_path;
        $params             = $this->params;
        $pm['dir']          = $params->get('mp3FTPDir', 'components/com_comprofiler/plugin/user/plug_cbmp3tab');
        $pm['dir_URL']      = $params->get('mp3Dir', 'components/com_comprofiler/plugin/user/plug_cbmp3tab');
        $pm['up_dir']       = $mosConfig_absolute_path."/".$pm['dir_URL']."/playlists";
        $pm['up_URL']       = $mosConfig_live_site."/".$pm['dir_URL']."/playlists";
        $pm['uploader']     = $params->get('mp3Uploader', '1');
        $pm['mp3max']       = intval($params->get('mp3max', '5'));
        $pm['show_URL'] 	= $params->get('mp3ShowURL', 'true');
        return $pm;
    }
    
    /**
	* Generates the HTML to display the user artist profile tab
	* @param object tab reflecting the tab database entry
	* @param object user reflecting the user currently being displayed
	* @param int 1 for front-end, 2 for back-end
	* @returns string return for tab content
	*/
	function getDisplayTab($tab,$user,$ui) {
        global $mosConfig_live_site, $mainframe, $my, $postdata, $database;
        $this->_getLanguageFile();
        $pm = $this->_getParams($user);
        if ($user->cb_mp3enable == "_UE_YES") {
        // Set up initial directory for MP3s and playlist...
            if (!file_exists($pm['up_dir'])) {
                do {
                    $dir = $pm['up_dir'];
                while (!@mkdir($dir,0777)) {
                    $dir = dirname($dir);
                    if ($dir == '/' || is_dir($dir))
                        break;
                    }
                } while ($dir != $pm['up_dir']);
            }
            if ($my->id == $user->id) { $isME = true; } else { $isME = false; }
            if ($isME && $pm['show_URL']) {
                $playlist = $pm['up_URL']."/".$user->id.".xml";
                $return = _UE_XSPF_PLAYLIST_URL."<a href=\"".$playlist."\"target=\"_blank\">".$playlist."</a><br />";
            }
            // Route form submits to upload and update functions.

            // Delete track and corresponding database row if it's called for...
            if ($delete = mosGetParam($_POST, 'delete')) {
                if ($delete != "keep" && $pm['delete'] ) {
                    if ($isME) {
                        $this->_deleteMP3($user, $delete, $pm['up_dir'], $pm['up_URL'], $postdata);
                    } else {
                        $message = _UE_XSPF_ACCESS_DENIED;
                        mosRedirect (sefRelToAbs("index.php?option=com_comprofiler&Itemid=".$Itemid),$message);
                    }
                }
            }
            
            $countquery = "SELECT id FROM #__comprofiler_plug_mp3 AS a WHERE a.user_id = $user->id";
    	    $database->setQuery($countquery);
    	    $mp3count = $database->loadResult();
    	    
    	    // Add a new track and update database
    		if ($isME && isset($_POST[userfileURL])) { // && $mp3count < $pm['mp3max']) {
                $file_URL = trim(strval($_POST[userfileURL]));
                $this->_updateData($user, $postdata, $file_URL);
                $this->_makePlaylist($user, $pm['up_dir'], $pm['up_URL'], $postdata);
                $message = _UE_XSPF_LINK_SUCCESS;
                mosRedirect (sefRelToAbs("index.php?option=com_comprofiler&Itemid=".$Itemid),$message);
            }

            // If params ask for the playlist manager, deliver it...
            if ($isME) { include('uploader.php'); }
            
			// Random number forces a reload of XML file.
			$rand = rand();
			// Display the XSPF player
            if($mp3count) {
                $params         = $this->params;
                $autoplay       = $params->get('mp3Autoplay', 'true');
                $player_title   = $params->get('mp3PlayerTitle', _UE_XSPF_PLAYER_TITLE);
                $swfurl = $mosConfig_live_site."/components/com_comprofiler/plugin/user/plug_cbmp3tab/xspf_player.swf?";
                $swfurl .= "playlist_url=".$pm['up_URL']."/".$user->id.".xml&";
                $swfurl .= "autoplay=".$autoplay."&autoload=true&player_title=".$player_title;
            // End options
    		    $return .= "<div class=\"tab_Description\">".unHtmlspecialchars(getLangDefinition($tab->description))."</div>\n";
            // Embedded player
                $return .= "<object classid=\"clsid:D27CDB6E-AE6D-11cf-96B8-444553540000\"";
                $return .= " codebase=\"http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=6,0,29,0\"";
                $return .= " width=\"368\" height=\"170\">\n";
                $return .= "<param name=\"movie\" value=\"".$mosConfig_live_site."/components/com_comprofiler/plugin/user/plug_cbmp3tab/xspf_player.swf?";
                $return .= "playlist_url=".$pm['up_URL']."/".$user->id.".xml?rand=".$rand."&";
                $return .= "autoplay=".$autoplay."&autoload=true&player_title=".$player_title."\">";
                $return .= "<param name=\"quality\" value=\"high\">\n";
                $return .= "<param name=\"BGCOLOR\" value=\"#FFFFFF\">\n";
                $return .= "<embed src=\"".$mosConfig_live_site."/components/com_comprofiler/plugin/user/plug_cbmp3tab/xspf_player.swf?";
                $return .= "playlist_url=".$pm['up_URL']."/".$user->id.".xml?rand=".$rand."&";
                $return .= "autoplay=".$autoplay."&autoload=true&player_title=".$player_title."\"";
                $return .= " width=\"368\" height=\"170\" quality=\"high\"";
                $return .= " pluginspage=\"http://www.macromedia.com/go/getflashplayer\" type=\"application/x-shockwave-flash\" bgcolor=\"#FFFFFF\">\n";
                $return .= "</embed>\n</object>\n";
            // Standalone player
                $return .= "<br /><input class=\"button\" type=\"button\" value=\""._UE_XSPF_STANDALONE."\" onclick=\"mp3Standalone('$swfurl');\" />";
              }
              return $return;
          }
    } // end getDisplayTab

    /**
	* Generates the HTML to display the artist user edit tab
	* @param object tab reflecting the tab database entry
	* @param object user reflecting the user being displayed
	* @param int 1 for front-end, 2 for back-end
	* @returns mixed : either string HTML for tab content, or false if ErrorMSG generated
	*/
    function getEditTab($tab,$user,$ui) {
        global $database, $mosConfig_absolute_path, $mosConfig_live_site;
        $this->_getLanguageFile();
        $pm = $this->_getParams($user);
    
        // Flash may be able to support .ogg Vorbis files someday...wishful thinking...
        $file_extensions        = array('.mp3');
        $message                = "";
    } // end class getEditTab

    /**
	* Deletes and logs MP3 uploads or links
	* @param object user
	* @param string delete database id of the MP3
	* @param string upload_directory route to the servers upload path
	* @param string upload_uri route to the same path as a URL
	* @param string name of the file to be uploaded
	* @param string the upload directory plus validated file name
	* @returns boolean : true by default, false or mosRedirect to error if the routine is not completed
	*/
    function _deleteMP3($user, $delete, $upload_directory, $upload_uri, $postdata) {
        global $database, $mosConfig_live_site, $mosConfig_absolute_path;
        $delquery1 = "SELECT location FROM #__comprofiler_plug_mp3 WHERE id = $delete";
        $database->setQuery( $delquery1 );
	    $del1 = $database->loadResult();
	    $del1 = str_replace($mosConfig_live_site, $mosConfig_absolute_path, $del1);
        if (@unlink( $del1 ) || !is_file( $del1 )) {
            $delquery2 = "DELETE FROM #__comprofiler_plug_mp3 WHERE id = $delete";
            $database->setQuery( $delquery2 );
            $database->query( $delquery2 );
            // $resource = @fopen($mosConfig_live_site."/components/com_comprofiler/plugins/user/plug_cbmp3tab/upload_log.txt","a+");
            // fwrite($resource,date("Y-F-d / h:i:sa")." - ".$del1." "._UE_XSPF_DELETED_BY." ".$user->username."\n");
            // fclose($resource);
            
            // Rebuild the playlist after deleting...
            $this->_makePlaylist($user, $upload_directory, $upload_uri, $postdata);
            $message = _UE_XSPF_DELETED_SUCCESS;
            mosRedirect (sefRelToAbs("index.php?option=com_comprofiler&Itemid=".$Itemid),$message);
        } else {
            $message = _UE_XSPF_DELETED_FAIL;
            mosRedirect (sefRelToAbs("index.php?option=com_comprofiler&Itemid=".$Itemid),$message);
        }
    }
    
    /**
	* Updates MP3 info to the comprofiler_plug_mp3 data table
	* @param object current user
	* @param string route to the upload directory of the file uploaded
	* @param string same route as a URL
	* @param string _POST data fields needed to populate the table
	* @param string permanent URL of the MP3
	* @returns boolean : true by default, false or mosRedirect to error message if the routine is not completed
	*/
    function _updateData($user, $postdata, $file_URL) {
        global $database, $mainframe, $mosConfig_live_site, $mosConfig_absolute_path, $Itemid;
        require_once( $mosConfig_absolute_path."/components/com_comprofiler/plugin/user/plug_cbmp3tab/xspf.class.php" );

        // External URL validation - use libcURL just in case allow_url_fopen isn't on
        $response = $this->processURL($file_URL);
        // die (print_r($response));
        
        // $message .= _UE_XSPF_FILE_VALIDATED;
        if (!$response || !eregi("(.)+\\.(mp3)",$file_URL)) {
            $message = $file_URL.": "._UE_XSPF_FILE_INVALID;
            mosRedirect(sefRelToAbs("index.php?option=com_comprofiler&Itemid=".$Itemid),$message);
        //  }   // else if (strpos($response[0], "200") === false) { // || (strpos($response[Content-Type], "/mpeg") === false)) {
            //   $message = $file_URL.": "._UE_XSPF_FILE_INVALID;
            //   $message .= "<br /> Debug:".print_r($response);
            // die(print_r($response[Content-Type]));
            //  mosRedirect(sefRelToAbs("index.php?option=com_comprofiler&Itemid=".$Itemid),$message);
        } else {
            $xspfdata['location'] = $file_URL;
        }

        // Artist and photo URL validation...could use more...
        $artistURL = mosGetParam( $_POST, 'artistURL', '' );
        if (!eregi('http://', $artistURL)) {
            $artistURL = "http://".$artistURL;
        }

        $photoURL = mosGetParam( $_POST, 'photoURL', '' );
        if ($photoURL) {
            if (!eregi("(.)+\\.(jp(e){0,1}g$)",$photoURL)){
                $photoURL = "";
                $message = _UE_XSPF_PHOTO_INVALID;
                mosRedirect(sefRelToAbs("index.php?option=com_comprofiler&Itemid=".$Itemid),$message);
            }
            if (!eregi('http://', $photoURL)) {
                $photoURL = "http://".$photoURL;
            }
        }

        // load up the form data
        $xspfdata['user_id']    = $user->id;
        $xspfdata['artist']     = mosGetParam( $_POST, 'mp3artist', '' );
        $xspfdata['title']      = mosGetParam( $_POST, 'mp3title', '' );
        $xspfdata['info']       = $artistURL;
        $xspfdata['photo']      = $photoURL;
        
        $xspf = new xspf($database);

        // bind it to the table
        if (!$xspf -> bind($xspfdata)) {
            $message = $database->getErrorMsg();
            mosRedirect (sefRelToAbs("index.php?option=com_comprofiler&Itemid=".$Itemid),$message);
		    exit();
	   }

	    // store it in the db
	    if (!$xspf -> store()) {
		   $message = $database->getErrorMsg();
           mosRedirect (sefRelToAbs("index.php?option=com_comprofiler&Itemid=".$Itemid),$message);
		   exit();
	    }
    } // end _updateData function

    /**
	* Generates XSPF playlist from data table
	* @param object current user
	* @param string route to the upload directory of the file uploaded
	* @param string same route as a URL
	* @param string _POST data fields needed to populate the table
	* @returns boolean : true by default, false or mosRedirect to error if the routine is not completed
	*/
    function _makePlaylist($user, $upload_directory, $upload_uri, $postdata) {

       global $mosConfig_live_site, $mosConfig_absolute_path, $database;
       require_once( $mosConfig_absolute_path."/components/com_comprofiler/plugin/user/plug_cbmp3tab/xspf.class.php" );

       $xspfquery = "SELECT *"
	   . " FROM #__comprofiler_plug_mp3 AS a"
	   . " WHERE a.user_id = $user->id"
	   . " ORDER BY a.artist ASC, a.title ASC"
	   ;
	   $database->setQuery( $xspfquery );
	   $xspf = $database->loadObjectList();
       $playlistURL = $upload_directory."/".$user->id.".xml";
       if (is_dir($upload_directory)) {

            $i = 0;

            $playlist  = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n";
            $playlist .= "\t<playlist version=\"0\" xmlns=\"http://xspf.org/ns/1/\">\n";
            $playlist .= "\t<title>" . _UE_XSPF_PLAYLIST_TITLE." " . $user->username . "</title>\n";
            $playlist .= "\t<annotation>" . $user->username . "</annotation>\n";
            $playlist .= "\t<creator>" . $user->username . "</creator>\n";
            $playlist .= "\t<info>" . $user->id . "</info>\n";
            $playlist .= "\t<location>" .$upload_uri. "</location>\n";
            $playlist .= "\t<trackList>\n";
            /* might need this later for getID3 capability
            if ($handle = opendir($upload_directory)) {

                while (false !== ($file = readdir($handle)))
                {
                    if ($file == ".") { $file = ""; }
                    elseif ($file == "..") { $file = ""; }
                    elseif (substr($file,strlen($file)-4,4) != ".mp3") { $file = ""; }
                    else {
                        $audiofile[$i] = $file;
                        $i++;
                    }
                }

            closedir($handle);
            }
        */
        // sort($audiofile);
        //this has to be moved ahead of the query...part of user parameters for the player?
        //if ($reverse == true) {
        //    rsort($audiofile);
        //    }
        //if ($shuffle == true) {
        //   shuffle($audiofile);
        //}

        //reset($audiofile);
        $songtitle = "";

        foreach($xspf as $xspfrow) {
            $songtitle = eregi_replace("_", " ", $xspfrow->title);
            $songtitle = eregi_replace("-", " ", $songtitle);
            $playlist .= "\t\t<track>\n";
            //$playlist .= "\t\t\t<location>" . $upload_uri ."/". $audiofile[$j] . "</location>\n";
            $playlist .= "\t\t\t<location>" . $xspfrow->location . "</location>\n";
            /* Next version...Flash player needs a little tweaking for user-defined download parameters.
            if ($download) {
                $playlist .= "\t\t\t<download>" . $upload_uri ."/". $audiofile[$j] . "</download>\n";
            }
            */
            $playlist .= "\t\t\t<image>" . $xspfrow->photo . "</image>\n";
            $playlist .= "\t\t\t<annotation>" . $xspfrow->artist . " - " . $songtitle . "</annotation>\n";
            $playlist .= "\t\t\t<info>" .$xspfrow->info . "</info>\n";
            //$playlist .= "\t\t\t<meta rel=" . $mosConfig_live_site . "\">$xspfrow->id</meta>";
            $playlist .= "\t\t</track>\n";
            $j++;
        }
        $playlist .= "\t</trackList>\n</playlist>\n";
        // die("Got to the playlist opener...");
        // Write the playlist file...
        $playlistFile = @fopen($playlistURL, "wb");
        if($playlistFile) {
            fwrite($playlistFile, $playlist);
	        fclose($playlistFile);
        } else {
            $message = $songtitle.": "._UE_XSPF_PLAYLIST_CREATE_ERROR;
            mosRedirect (sefRelToAbs("index.php?option=com_comprofiler&Itemid=".$Itemid),$message);
        }
	    // return true;
    } else {
        $message = $playlist." "._UE_XSPF_PLAYLIST_DIR_ERROR;
        $message .= "\n". $playlistURL;
        mosRedirect (sefRelToAbs("index.php?option=com_comprofiler&Itemid=".$Itemid),$message);
    }
} // end function _makePlayList
    
    
    function processURL($url){
        // For those still in PHP 4...
        $headers = "";
        if(!function_exists('get_headers')) {
            $headers = $this->get_headers_4($url,1);
        } else {
            $headers = @get_headers($url, 1);
        }
        // die(print_r($headers));
        if ($headers) {return $headers;} else {return false;}
    }
   
   function get_headers_4($url,$format=0) {
       $url=parse_url($url);
       $end = "\r\n\r\n";
       $fp = fsockopen($url['host'], (empty($url['port'])?80:$url['port']), $errno, $errstr, 30);
       if ($fp)
       {
           $out  = "GET / HTTP/1.1\r\n";
           $out .= "Host: ".$url['host']."\r\n";
           $out .= "Connection: Close\r\n\r\n";
           $var  = '';
           fwrite($fp, $out);
           while (!feof($fp))
           {
               $var.=fgets($fp, 1280);
               if(strpos($var,$end))
                   break;
           }
           fclose($fp);

           $var=preg_replace("/\r\n\r\n.*\$/",'',$var);
           $var=explode("\r\n",$var);
           if($format)
           {
               foreach($var as $i)
               {
                   if(preg_match('/^([a-zA-Z -]+): +(.*)$/',$i,$parts))
                       $v[$parts[1]]=$parts[2];
               }
               return $v;
           }
           else
               return $var;
       }
   }

} // end class getMp3Tab
?>
